 % Using the Sample Data, 
        load '../Day07/ERN_Data_LM_Ref.mat';  % provided with Day07 download, but change as needed

        % uncomment to run for laplacian
%         EEG.data = laplacian_perrinX(EEG.data,[EEG.chanlocs.X],[EEG.chanlocs.Y],[EEG.chanlocs.Z]);

    % explore the time and topography to choose a channel to focus on
        pop_topoplot(EEG,1, [50:50:800] ,'EEG sample dataset',[4 4] ,0,'electrodes','on');

%% Setup TF stuff

    % choose a site 
        channel2use = 'FCz';  
        chan2use_idx = strcmpi(channel2use,{EEG.chanlocs.labels});
        
    % times to plot (avoid edge effects)
        times2plot = [-100 800]; % or choose your own
        times2plot_idx = dsearchn(EEG.times',times2plot')';
        times2plot_idx = times2plot_idx(1):times2plot_idx(2);
        
    % Using the ERN sample data at this site                                
        % compute the time-frequency power  and ITPC on the raw data
        
        % wavelet parameters
            num_frex = 20;
            min_freq =  2;
            max_freq = 15;

        % set range for variable number of wavelet cycles
            range_cycles = [ 4 10 ];

        % more wavelet parameters
            frex = logspace(log10(min_freq),log10(max_freq),num_frex);
            wavecycles = logspace(log10(range_cycles(1)),log10(range_cycles(end)),num_frex);
            time = -2:1/EEG.srate:2;
            half_wave_size = (length(time)-1)/2;

        % FFT parameters
            nWave = length(time);
            nData = EEG.pnts*EEG.trials;
            nConv = nWave+nData-1;

    %% time-frequency decomposition
        % FFT of data (doesn't change on frequency iteration)
            dataX = fft( reshape(EEG.data(chan2use_idx,:,:),1,nData) ,nConv);

        % initialize output time-frequency data
            tf = zeros(num_frex,EEG.pnts,EEG.trials);
            tf_power = zeros(num_frex,EEG.pnts);
            tf_phase = zeros(num_frex,EEG.pnts);

        % loop over frequencies
        for fi=1:num_frex

            % create wavelet and get its FFT
            s = wavecycles(fi)/(2*pi*frex(fi));
            wavelet  = exp(2*1i*pi*frex(fi).*time) .* exp(-time.^2./(2*s^2));
            waveletX = fft(wavelet,nConv);

            % run convolution
            as = ifft(waveletX.*dataX,nConv);
            as = as(half_wave_size+1:end-half_wave_size);
            as = reshape(as,EEG.pnts,EEG.trials);

            % put this into the tf matrix of complex numbers
            tf(fi,:,:) = as;
        end
        
        
        % compute ITPC  (it's the absolute length of the mean Eulerized phase
            % angle across trials: abs, mean, euler, angle)
              % For each frequency, this will be a time-series of ITPC values 
            tf_phase= abs(mean(exp(1i*angle(tf)),3));
            
        % computer power at each freq and time (it's the mean of the length
            % of the complex vector (squared for power)
            % mean across trials, abs, dot-squared
            tf_power= mean(abs(tf).^2,3);
            
        % baseline correct the data using db-correction
            baseline_window = [ -300 -100] ; % in msec -- choose your own!
        % convert baseline time into indices
            baseidx = dsearchn(EEG.times',baseline_window')';
            baseline = mean( tf_power(:,baseidx(1):baseidx(2)) ,2);
        % decibel correction
            tf_power = 10*log10(bsxfun(@rdivide, tf_power, baseline) );
    
%%  Plotting  
        % Plot a 1 x 2 set of subpots with power, ITPC, but place it in a
        % 3x2 grid of subplots (as the first row of this bigger grid)
            
    dblim = [-5 5];
    itpclim = [0 .7];

    figure(Name='Voltage Data');  
    % uncomment next if you rerun with laplacian transorm
%     figure(Name='Laplacian Data');  
    
    subplot(321); % Channel 1 Power
    contourf(EEG.times,frex,tf_power,40,'linecolor','none');
    set(gca,'clim',dblim,'ydir','normal','xlim',[-300 800])
    title(['TF Power -- dB baseline for channel '  channel2use]);
    subplot(322); % Channel 1 Phase
    contourf(EEG.times,frex,tf_phase,40,'linecolor','none')
    set(gca,'clim',itpclim,'ydir','normal','xlim',[-300 800])
    title(['TF ITPC for channel '  channel2use]);
   
    %% ASSIGNMENT: Do the same for a second site        
        % Do this again for a second site
        % save the AS data in a different variable for later phase angle diffs calculations
        tf_channel1 = tf;
        channel1 = channel2use;

% ======================
% repeat of above text with new channel
% ======================
    % choose a site 
        channel2use = 'P3';  
        chan2use_idx = strcmpi(channel2use,{EEG.chanlocs.labels});
        
    % Using the ERN sample data at this site                                
        % compute the time-frequency power  and ITPC on the raw data
        
        % wavelet parameters
            num_frex = 20;
            min_freq =  2;
            max_freq = 15;

        % set range for variable number of wavelet cycles
            range_cycles = [ 4 10 ];

        % more wavelet parameters
            frex = logspace(log10(min_freq),log10(max_freq),num_frex);
            wavecycles = logspace(log10(range_cycles(1)),log10(range_cycles(end)),num_frex);
            time = -2:1/EEG.srate:2;
            half_wave_size = (length(time)-1)/2;

        % FFT parameters
            nWave = length(time);
            nData = EEG.pnts*EEG.trials;
            nConv = nWave+nData-1;

    %% time-frequency decomposition
        % FFT of data (doesn't change on frequency iteration)
            dataX = fft( reshape(EEG.data(chan2use_idx,:,:),1,nData) ,nConv);

        % initialize output time-frequency data
            tf = zeros(num_frex,EEG.pnts,EEG.trials);
            tf_power = zeros(num_frex,EEG.pnts);
            tf_phase = zeros(num_frex,EEG.pnts);

        % loop over frequencies
        for fi=1:num_frex

            % create wavelet and get its FFT
            s = wavecycles(fi)/(2*pi*frex(fi));
            wavelet  = exp(2*1i*pi*frex(fi).*time) .* exp(-time.^2./(2*s^2));
            waveletX = fft(wavelet,nConv);

            % run convolution
            as = ifft(waveletX.*dataX,nConv);
            as = as(half_wave_size+1:end-half_wave_size);
            as = reshape(as,EEG.pnts,EEG.trials);

            % put this into the tf matrix of complex numbers
            tf(fi,:,:) = as;
        end
        
        
        % compute ITPC  (it's the absolute length of the mean Eulerized phase
            % angle across trials: abs, mean, euler, angle)
              % For each frequency, this will be a time-series of ITPC values 
            tf_phase= abs(mean(exp(1i*angle(tf)),3));
            
        % computer power at each freq and time (it's the mean of the length
            % of the complex vector (squared for power)
            % mean across trials, abs, dot-squared
            tf_power= mean(abs(tf).^2,3);
            
        % baseline correct the data using db-correction
            baseline_window = [ -300 -100] ; % in msec -- choose your own!
        % convert baseline time into indices
            baseidx = dsearchn(EEG.times',baseline_window')';
            baseline = mean( tf_power(:,baseidx(1):baseidx(2)) ,2);
        % decibel correction
            tf_power = 10*log10(bsxfun(@rdivide, tf_power, baseline) );

% ======================
% repeat of above text with new channel
% ======================

        % Plot in the second row of subpllots (power, ITPC)
            
    dblim = [-5 5];
    itpclim = [0 .7];

    subplot(323); % Channel 1 Power
    contourf(EEG.times,frex,tf_power,40,'linecolor','none');
    set(gca,'clim',dblim,'ydir','normal','xlim',[-300 800])
    title(['TF Power -- dB baseline for channel '  channel2use]);
    subplot(324); % Channel 1 Phase
    contourf(EEG.times,frex,tf_phase,40,'linecolor','none')
    set(gca,'clim',itpclim,'ydir','normal','xlim',[-300 800])
    title(['TF ITPC for channel '  channel2use]);
   
        
   %% ASSIGNMENT: Now get ISPC bewtween the two sites, 
    %   and calculate ISPC Value and (BONUS) PLI Value
        % need to get phase angle differences
        
        % plot ISPC in the 6th subplot and include title to show ISPC value
        % (and PLI if you feel BONUSy)

        % copy channel 2 as for keeping names clear
        tf_channel2 = tf;
        channel2 = channel2use;
        ispclim = [0 .8];
        plilim = [0 .5];


        % get phase angle diffs
        phasechannel1 = angle(tf_channel1);
        phasechannel2 = angle(tf_channel2);
        phase_diffs = phasechannel1 - phasechannel2;

        phase_synchronization = abs(mean(exp(1i*phase_diffs),3));
        
        % plot in the 6th subplot
        subplot(326)
        contourf(EEG.times,frex,phase_synchronization,40,'linecolor','none')
        set(gca,'clim',ispclim,'ydir','normal','xlim',[-300 800])
        title(['TF ISPC for channels '  channel1 ' and ' channel2]);

        % plot PLI (BONUS) in 5th subplot
        pli  = squeeze(abs(mean(sign(imag(exp(1i*phase_diffs))),3)));
        subplot(325)
        contourf(EEG.times,frex,pli,40,'linecolor','none')
        set(gca,'clim',plilim,'ydir','normal','xlim',[-300 800])
        title(['TF PLI for channels '  channel1 ' and ' channel2]);

  
    %% ASSIGNMENT:  Now redo all the above for laplacian-transformed data, 
    % and plot in a new figure so you can compare voltage to CSD data

    % Uncomment the two lines to transform the data and give the figure a
    % new name

    
